## Reconstruct backbone from PDB ##
#!/usr/bin/env Rscript
args = commandArgs(trailingOnly=TRUE)
# Two user inputs: 1) Sequence file, 2) Folder with MCDNA pdb's (ohne output_pdb)

#library("BioPhysConnectoR")
library("bio3d")	# works only for R versions higher than 3.1, otherwise use BioPhysConnectoR with R from orozco36

source("bkb_conf_helpars_function.R")	# Function to predict bkb state

## Functions ##
vcrossp <- function(a,b){
result <- rep(NA,3)
result[1] <- a[2] * b[3] - a[3] * b[2]
result[2] <- a[3] * b[1] - a[1] * b[3]
result[3] <- a[1] * b[2] - a[2] * b[1]
result
}

vec_len = function(v){
len = sqrt(v[1]*v[1]+v[2]*v[2]+v[3]*v[3])
len
}

rec_atm = function(p,at1,at2,at3)	#p: 1st row: position in ref cosy (system 1), 2nd to 4th row: axes of ref cosy
{
#Aufpunkt: at1

xax = (at2-at1)/vec_len(at2-at1)
zax = vcrossp(xax,at3-at1)/vec_len(vcrossp(xax,at3-at1))
yax = vcrossp(xax,zax)/vec_len(vcrossp(xax,zax))
# New coordinate tranformation of system 2 with respect to global system
Q2 = rbind(xax,yax,zax)

vecref = p	# Vector in internal coordinates of base in example structure

vres = vecref[1] * xax + vecref[2] * yax + vecref[3] * zax + at1

return(vres)
}

cart2sph = function(x)
{
r = sqrt(x[1]*x[1] + x[2]*x[2] + x[3]*x[3])
th = acos(x[3] / r)
ph = atan2(x[2], x[1])
return(c(r,th,ph))
}

sph2cart = function(x)
{
X = x[1] * sin(x[2]) * cos(x[3])
Y = x[1] * sin(x[2]) * sin(x[3])
Z = x[1] * cos(x[2])
return(c(X,Y,Z))
}

## End Functions ##

##### START MAIN #####

# User Input
seq_file = as.character(args[1])
folder = as.character(args[2])
num_strucs = as.numeric(as.character(args[3]))
#print(seq_file)
#print(folder)
#print(num_strucs)
#seq_file = "/usr/people/jwalther/Programs/DNA_multimodal/src/seq_nl15.txt"
#folder = "/orozco/projects/MontecarloJurgen/data/mcdna_performance_hpc/nl15"

#seq_file = "/usr/people/jwalther/Programs/DNA_multimodal/src/seq_56merTIP3P.dat"
#folder = "/orozco/projects/MontecarloJurgen/data/adj_angle_exact_Ph_pos_56TIP3P/bsc1_avg_k_miniabc_multi"
#num_strucs = 50000

# End User Input
folder_templates = "Data/bkb_state/avg_bkb_state/Rdata"
files_temp_full = list.files(folder_templates, full.names=T)
files_temp = list.files(folder_templates, full.names=F)
base = unlist(lapply(files_temp, function(i) substr(i, start=5, stop=5)))
dimer = unlist(lapply(files_temp, function(i) substr(i, start=5, stop=6)))
dimer_bkbstate = lapply(files_temp_full, function(x) {load(x)
					 return(out)
					}
	   )

p_all_b1_c = get(load("Data/bkb_state/10_C5_507550_BI.pdb.Rdata"))
p_all_b2_c = get(load("Data/bkb_state/10_C5_503596_BII.pdb.Rdata"))

p_all_b1_t = get(load("Data/bkb_state/1_T10_501450_BI.pdb.Rdata"))
p_all_b2_t = get(load("Data/bkb_state/1_T10_507148_BII.pdb.Rdata"))

p_all_b1_a = get(load("Data/bkb_state/2_A10_522077_BI.pdb.Rdata"))
p_all_b2_a = get(load("Data/bkb_state/2_A10_501932_BII.pdb.Rdata"))

p_all_b1_g = get(load("Data/bkb_state/1_G8_508821_BI.pdb.Rdata"))
p_all_b2_g = get(load("Data/bkb_state/1_G8_500175_BII.pdb.Rdata"))

#filename = "/orozco/projects/MontecarloJurgen/data/56merTIP3P/bsc1_k_avg_miniabc/output_pdb/structure_000000.pdb"

#folder = "/orozco/projects/MontecarloJurgen/data/miniabc_mcdna/multimodal/13"
files = list.files(paste0(folder,"/output_pdb"), full.names=TRUE, pattern="\\.pdb$")	# PDB files
files_names = list.files(paste0(folder,"/output_pdb"), full.names=FALSE, pattern="\\.pdb$")
files_name = unlist(lapply(files_names, function(i) unlist(strsplit(i,".",fixed=TRUE))[1] ))

#seq_file = "~/Programs/DNA_multimodal/src/13_miniabc.dat"
first_snap = 1	# First snapshot
last_snap = num_strucs	# Last snapshot
pred_bkb_states = correct_bkb_states(seq_file,folder,first_snap,last_snap)
save(pred_bkb_states, file=paste0(folder,"/pred_bkb_states.Rdata") )

single_pdb = function(ind,p_all_b2_c,p_all_b2_t,p_all_b2_a,p_all_b2_g)
{
filename = files[ind]
filnm = files_name[ind]
#print(ind)
pdb_buf = read.pdb(filename,verbose=FALSE) # R/3.2.0-goolf-1.6.10 (from gpucluster)
pdb = as.data.frame(pdb_buf$atom[,2:11], stringsAsFactors=FALSE)

# With BioPhysConnectoR 
#pdb_buf = extractPDB(filename,verbose=FALSE) # to get right x,y,z coordinates
#pdb = as.data.frame(pdb_buf$pdb$atom[,1:10], stringsAsFactors=FALSE)

pdb_ph = pdb[(pdb$elety %in% "P"),] 

if("P" %in% pdb$elety) pdb = pdb[!(pdb$elety %in% "P"),] # Delete Phosphates reconstructed by former algorithms

pdb$eleno = as.numeric(pdb$eleno)
pdb$resno = as.numeric(pdb$resno)
pdb$x = as.numeric(pdb$x)
pdb$y = as.numeric(pdb$y)
pdb$z = as.numeric(pdb$z)

len = pdb$resno[length(pdb$chain)]/2 # length of sequence

sequence = c(pdb$resid[unlist(lapply(1:(length(pdb$resid)-1), function(x) pdb$resno[x] != pdb$resno[x+1]))],pdb$resid[length(pdb$resid)])

real_seq = unlist(lapply(1:(length(sequence)-1), function(i) paste(substr(sequence[i], start=2, stop=2),substr(sequence[i+1], start=2, stop=2),collapse="",sep="")))
real_seq[length(sequence)] = "CG"



pdb_bkb_state = pred_bkb_states[[ind]]

vec_pdb_bkb_state = c(TRUE,pdb_bkb_state[,1],TRUE,TRUE, TRUE,rev(pdb_bkb_state[,2]),TRUE,TRUE)	# 1,len-1,len,len+1,2*len-1,2*len have no definition
#vec_pdb_bkb_state = c(TRUE,pdb_bkb_state[,1],TRUE,TRUE, TRUE, TRUE, rev(pdb_bkb_state[,2]),TRUE)	# 1,len-1,len,2*len-1,2*len have no definition	(sh1C)



#out_pdb = data.frame()

at1 = c(0,0,0)
at2 = c(0,0,0)
at3 = c(0,0,0)
p_all_b1 = p_all_b1_c
p_all_b2 = p_all_b2_c
bkb_state = TRUE
p_all = p_all_b1

rec_bkb = function(resno,p_all_b2_c,p_all_b2_t,p_all_b2_a,p_all_b2_g)
{

#print(resno)
#for(resno in 1:max(pdb$resno))
#{

pdb_base = pdb[pdb$resno == resno,]

pdb_c = pdb_base[pdb_base$resno == pdb_base$resno[1],]
base = pdb_base$resid[1]
#print(base)
#print(resno)
#base
n3 = as.numeric(pdb_c[(pdb_c$elety == "N3") ,8:10])
n4 = as.numeric(pdb_c[(pdb_c$elety == "N4") ,8:10])
c4 = as.numeric(pdb_c[(pdb_c$elety == "C4") ,8:10])
n1 = as.numeric(pdb_c[(pdb_c$elety == "N1") ,8:10])
o2 = as.numeric(pdb_c[(pdb_c$elety == "O2") ,8:10])
c6 = as.numeric(pdb_c[(pdb_c$elety == "C6") ,8:10])
c8 = as.numeric(pdb_c[(pdb_c$elety == "C8") ,8:10])
c5 = as.numeric(pdb_c[(pdb_c$elety == "C5") ,8:10])
n9 = as.numeric(pdb_c[(pdb_c$elety == "N9") ,8:10])


# For C: n1,n4,o2; T: n1,c4,c6; A: n9,c8,c5; G: n9,c8,c5
if(base == "DC"){
at1 = n1
at2 = n4
at3 = o2

p_all_b1 = dimer_bkbstate[[which(dimer == real_seq[resno])[2]]]
p_all_b2 = dimer_bkbstate[[which(dimer == real_seq[resno])[1]]]
		}

if(base == "DT"){
at1 = n1
at2 = c4
at3 = c6
p_all_b1 = dimer_bkbstate[[which(dimer == real_seq[resno])[2]]]
p_all_b2 = dimer_bkbstate[[which(dimer == real_seq[resno])[1]]]
		}

if(base == "DA"){
at1 = n9
at2 = c8
at3 = c5
p_all_b1 = dimer_bkbstate[[which(dimer == real_seq[resno])[2]]]
p_all_b2 = dimer_bkbstate[[which(dimer == real_seq[resno])[1]]]
		}

if(base == "DG"){
at1 = n9
at2 = c8
at3 = c5
p_all_b1 = dimer_bkbstate[[which(dimer == real_seq[resno])[2]]]
p_all_b2 = dimer_bkbstate[[which(dimer == real_seq[resno])[1]]]
		}


if(resno %in% c(1,len+1,2*len)) bkb_state = TRUE else bkb_state = vec_pdb_bkb_state[resno]	# Assigns backbone state

bkb_state = !bkb_state	# Invert to have it clear for the new classification (6.12.18)

if(bkb_state == TRUE) p_all = p_all_b1 else p_all = p_all_b2	# BI = TRUE; BII = FALSE


res = do.call("rbind",lapply(1:length(p_all[[1]][,1]), function(i) rec_atm(as.numeric(p_all[[1]][i,]),at1,at2,at3)))



pdb_res_add = cbind(c((pdb_base$eleno[1]-length(p_all[[2]])+1):pdb_base$eleno[1]-1),p_all[[2]],rep(pdb_base$alt[1],length(p_all[[2]])),rep(pdb_base$resid[1],length(p_all[[2]])),rep(pdb_base$chain[1],length(p_all[[2]])),rep(pdb_base$resno[1],length(p_all[[2]])),rep(pdb_base$insert[1],length(p_all[[2]])),round(res,3))

pdb_res_add[(length(pdb_res_add[,1])-3):length(pdb_res_add[,1]),6] = as.character(rep(as.numeric(pdb_res_add[1,6])+1,4))	# change resid of bkb atoms belonging to base i+1
pdb_res_add[(length(pdb_res_add[,1])-3):length(pdb_res_add[,1]),4] = rep(sequence[pdb_base$resno[1]+1],4)

pdb_res_add = as.data.frame(pdb_res_add)
colnames(pdb_res_add) = colnames(pdb_base)
pdb_res_add[,1] = as.numeric(as.character(pdb_res_add[,1]))
pdb_res_add[,6] = as.numeric(as.character(pdb_res_add[,6]))
pdb_res_add[,8] = as.numeric(as.character(pdb_res_add[,8]))
pdb_res_add[,9] = as.numeric(as.character(pdb_res_add[,9]))
pdb_res_add[,10] = as.numeric(as.character(pdb_res_add[,10]))

pdb_res_add_cart = pdb_res_add[,8:10]

# Move reconstructed part to formerly determined (Lavery) P position 
if(!(resno %in% c(1,len,2*len))){
ph_pos = as.numeric(unlist(pdb_ph[as.numeric(pdb_ph$resno) == resno+1, 8:10]))
pdb_phpos = unlist(pdb_res_add[pdb_res_add$elety == "P",8:10])
mv_ph = ph_pos - pdb_phpos

buf = do.call("rbind",lapply(1:length(pdb_res_add_cart[,1]), function(x) pdb_res_add_cart[x,] + mv_ph))	# Translation to new P position ph_pos


buf_centph = do.call("rbind",lapply(1:length(buf[,1]), function(x) buf[x,] - ph_pos))	# Put coordinates in reference to new P positions ph_pos
buf_formercart_centph = do.call("rbind",lapply(1:length(pdb_res_add_cart[,1]), function(x) pdb_res_add_cart[x,] - ph_pos))

c1p = as.numeric(buf_centph[4,])	# Get coordinates of C1'
c1pf = as.numeric(buf_formercart_centph[4,])	# Get coordinates of former C1'

# Find rotation matrix from v1 to v2 ( B = U A)
find_rotmat = function(v1,v2){

A = v1/vec_len(v1)
B = v2/vec_len(v2)

ABdot  = as.numeric(A %*% B)
ABcrs  = vcrossp(A,B)
ABcra  = vec_len(ABcrs)

G = matrix(0,3,3)
Fm = matrix(0,3,3)
  G[3,3] = 1
  G[1,1] = ABdot
  G[2,2] = ABdot
  G[2,1] = ABcra
  G[1,2] = -ABcra

  Fm[1,]   = A	#maybe this is for a column
  Fm[2,]   = (B-ABdot*A)/vec_len(B-ABdot*A)
  Fm[3,]   = -ABcrs/ABcra

  U = (t(Fm) %*% G) %*% Fm

  return(U)
}

R = find_rotmat(c1p,c1pf)	# Rotation matrix


rotate_coords = function(i,df)	# Rotate coordinates of rigid body backbone
{
#print(i)
x = unlist(df[i,])
if(x[1] == 0 & x[2] == 0 & x[3] == 0) return(c(0,0,0)) else{
cart_new = as.numeric(R %*% x)
return(cart_new)}
}

buf_centph_new = do.call("rbind", lapply(1:length(buf_centph[,1]), rotate_coords, buf_centph))

buf_new = round(do.call("rbind",lapply(1:length(buf_centph_new[,1]), function(x) buf_centph_new[x,] + ph_pos)),3)


#vis = buf_centph
#scatter3D(vis[,1], vis[,2], vis[,3],colvar = vis[,3], col = NULL, add = FALSE, phi = 0)
#text3D(vis[,1], vis[,2], vis[,3],  labels = pdb_res_add$elety, add = TRUE, colkey = FALSE, cex = 2)

pdb_res_add[,8:10] = buf_new

}
# END move


pdb_res_bef = pdb_res_add[pdb_res_add[,2] %in% c("C5'","C4'","O4'","C1'"),]
pdb_res_aft = pdb_res_add[pdb_res_add[,2] %in% c("C3'","C2'","O3'","P","OP1","OP2","O5'"),]

pdb_out = rbind(pdb_res_bef,pdb_base,pdb_res_aft)

if(resno %in% c(1,len,len+1)) pdb_out = pdb_base	# Terminating bases do have "open" backbone, so no need for reconstruction
if(resno %in% c(2*len)) pdb_out = rbind(pdb_res_bef,pdb_base,pdb_res_add[pdb_res_add[,2] %in% c("C3'","C2'","O3'"),])
	# Terminating bases do have "open" backbone, so no need for reconstruction

return(pdb_out)
#if(resno == 1) out_pdb = pdb_out else out_pdb = rbind(out_pdb,pdb_out)
}

out_pdb = do.call("rbind", lapply(1:max(pdb$resno), rec_bkb,p_all_b2_c,p_all_b2_t,p_all_b2_a,p_all_b2_g))
out_pdb$eleno = c(1:length(out_pdb$eleno))

#out_file=paste0("~/Programs/Bkb_states_miniabc/traj/",filnm,"_newTBII_rand_bkb.pdb")
out_file=paste0(folder,"/output_pdb/bkb/",filnm,"rec.pdb")
cat("HEADER  Nucleic Acids Structure Generated by Juergen Walther\n",file=out_file,sep="",append=FALSE)
cat("AUTHOR  Program written by Juergen Walther\n",file=out_file,sep="",append=TRUE)
for(i in 1:length(out_pdb[,1]))
{
if(i>1 && out_pdb$chain[i-1] == "A" && out_pdb$chain[i] == "B") cat(sprintf("TER\n"),file=out_file,sep="",append=TRUE) # To divide strands
cat(sprintf("ATOM  %5d  %-4s%s %s%+4s    %8.3f%8.3f%8.3f\n",as.integer(out_pdb$eleno[i]),out_pdb$elety[i],out_pdb$resid[i],out_pdb$chain[i],out_pdb$resno[i],as.numeric(out_pdb$x[i]),as.numeric(out_pdb$y[i]),as.numeric(out_pdb$z[i])),file=out_file,sep="",append=TRUE)
}


return(out_pdb)
}

out_pdbs = lapply(first_snap:last_snap,single_pdb,p_all_b2_c,p_all_b2_t,p_all_b2_a,p_all_b2_g)






