/* % ATOMIC_GLH build DNA atomic structure using GLOBAL parameters
   %
   % Input: nbp--number of base-pairs
   %        npseq--base-pair sequence
   %        bppar--[nbp by 6] CEHS base-pair parameters
   %        glhpar--[nbp by 6] GLOBAL parameters
   %        xdir--+x-axis direction */
void atomic_GLH(long nbp, char **bpseq, double **bppar, double **glhpar,
                long xdir, char *filnam)
{
    long i, j, k, m, i2, num1, num2, tn1 = 0, tn2 = 0;
    char base_set[BUF512], *dir_name, *env_add;
    double **org_xyz;
    char **asym1, **asym2, **t_asym1, **t_asym2;
    double **xyz1, **xyz2, **t_xyz1, **t_xyz2, **xyztmp1, **xyztmp2;
    char *t_btype1, *t_btype2;
    long *t_bnum1, *t_bnum2, ATM_NUM = 100;
    char bpname[3];
    long **ts2;  /* keep track of strand II base atoms */

    long *base_num, tot_num;
    char **tot_asym, *base_type, *strand;
    double **tot_xyz, **tot_xyz2;

    double twist_g = 0.0, rise_g = 0.0, tip, incl, y_disp, x_disp;
    double **v_orien, **v_orien_T, *bp_pos, *tmp_vec;
    double *hinge, TipInc, **bp_orien, **bp_orien_T;

    /* Read in atomic base geometry */
    dir_name = cvector(1, BUF512);

    printf("\nBase geometry set to use (Dft NDB96): ");
    read_stdin_str(base_set);

    if (strlen(base_set) == 0)
        strcpy(base_set, "NDB96");

    if ((env_add = getenv("SCHNA_AR_P")) == NULL)
        nrerror("Environment variable SCHNA_AR_P undefined!!!\n");
    strcpy(dir_name, env_add);
    strcat(dir_name, "/BaseGeo/");
    strcat(dir_name, base_set);
    strcat(dir_name, "_");

    /* Variable initializations */
    org_xyz = dmatrix(1, nbp, 1, 3);

    /* For strand I */
    t_asym1 = cmatrix(1, MAX_ATOM, 1, 4);
    t_btype1 = cvector(1, MAX_ATOM);
    t_bnum1 = lvector(1, MAX_ATOM);
    t_xyz1 = dmatrix(1, MAX_ATOM, 1, 3);

    /* For strand II */
    t_asym2 = cmatrix(1, MAX_ATOM, 1, 4);
    t_btype2 = cvector(1, MAX_ATOM);
    t_bnum2 = lvector(1, MAX_ATOM);
    t_xyz2 = dmatrix(1, MAX_ATOM, 1, 3);

    /* column 1: starting # for each base */
    /* column 2: total atom # for each base */
    ts2 = lmatrix(1, nbp, 1, 2);

    v_orien = dmatrix(1, 3, 1, 3);
    v_orien_T = dmatrix(1, 3, 1, 3);
    bp_orien = dmatrix(1, 3, 1, 3);
    bp_orien_T = dmatrix(1, 3, 1, 3);
    bp_pos = dvector(1, 3);
    tmp_vec = dvector(1, 3);
    hinge = dvector(1, 3);

    /* Get the whole data set by iterations */
    for (i = 1; i <= nbp; i++) {
        asym1 = cmatrix(1, ATM_NUM, 1, 4);
        asym2 = cmatrix(1, ATM_NUM, 1, 4);
        xyz1 = dmatrix(1, ATM_NUM, 1, 3);
        xyz2 = dmatrix(1, ATM_NUM, 1, 3);
        xyztmp1 = dmatrix(1, ATM_NUM, 1, 3);
        xyztmp2 = dmatrix(1, ATM_NUM, 1, 3);

        bpname[0] = bpseq[1][i - 1];
        bpname[1] = bpseq[2][i - 1];
        bpname[2] = '\0';

        set_bp(&num1, &num2, asym1, asym2, xyz1, xyz2, bpname, bppar[i], xdir, dir_name);

        /* Decompose the 6 GLOBAL parameters */
        x_disp = glhpar[i][1];
        y_disp = glhpar[i][2];
        rise_g += glhpar[i][3];
        incl = glhpar[i][4];
        tip = glhpar[i][5];
        twist_g += glhpar[i][6];

        rotz(v_orien, twist_g);  /* bp orientation after "twisting" */
        tr_dmatrix(v_orien_T, v_orien, 3, 3);
        tmp_vec[1] = x_disp;
        tmp_vec[2] = y_disp;
        tmp_vec[3] = rise_g;
        dvec_x_dmtx(bp_pos, tmp_vec, v_orien_T, 3, 3, 3);

        /* Each bp's origin in global reference frame */
        for (j = 1; j <= 3; j++)
            org_xyz[i][j] = bp_pos[j];

        for (j = 1; j <= 3; j++)
            hinge[j] = incl * v_orien[j][1] + tip * v_orien[j][2];  /* axis */
        TipInc = dist_ab(incl, tip);  /* angle */
        arbrot(v_orien_T, hinge, TipInc);  /* v_orien_T as rotmat */
        /* Final bp orientation */
        mul_dmatrix(bp_orien, v_orien_T, v_orien, 3, 3, 3, 3);
        tr_dmatrix(bp_orien_T, bp_orien, 3, 3);

        mul_dmatrix(xyztmp1, xyz1, bp_orien_T, num1, 3, 3, 3);
        for (j = 1; j <= num1; j++) {
            k = tn1 + j;
            for (m = 1; m <= 3; m++)
                t_xyz1[k][m] = xyztmp1[j][m] + bp_pos[m];
            strcpy(t_asym1[k], asym1[j]);
            t_btype1[k] = bpname[0];
            t_bnum1[k] = i;
        }

        ts2[i][1] = tn2 + 1;
        ts2[i][2] = num2;

        mul_dmatrix(xyztmp2, xyz2, bp_orien_T, num2, 3, 3, 3);
        for (j = 1; j <= num2; j++) {
            k = tn2 + j;
            for (m = 1; m <= 3; m++)
                t_xyz2[k][m] = xyztmp2[j][m] + bp_pos[m];
            strcpy(t_asym2[k], asym2[j]);
            t_btype2[k] = bpname[1];
            t_bnum2[k] = 2 * nbp - i + 1;
        }
        tn1 += num1;
        tn2 += num2;

        free_cmatrix(asym1, 1, ATM_NUM, 1, 4);
        free_cmatrix(asym2, 1, ATM_NUM, 1, 4);
        free_dmatrix(xyz1, 1, ATM_NUM, 1, 4);
        free_dmatrix(xyz2, 1, ATM_NUM, 1, 4);
        free_dmatrix(xyztmp1, 1, ATM_NUM, 1, 4);
        free_dmatrix(xyztmp2, 1, ATM_NUM, 1, 4);
    }

    tot_num = tn1 + tn2;

    tot_xyz = dmatrix(1, tot_num, 1, 3);
    tot_xyz2 = dmatrix(1, tot_num, 1, 3);
    tot_asym = cmatrix(1, tot_num, 1, 4);
    base_type = cvector(1, tot_num);
    strand = cvector(1, tot_num);
    base_num = lvector(1, tot_num);

    /* Combine two strands together! */
    for (i = 1; i <= tn1; i++) {
        strcpy(tot_asym[i], t_asym1[i]);
        base_type[i] = t_btype1[i];
        strand[i] = 'A';
        base_num[i] = t_bnum1[i];
        for (j = 1; j <= 3; j++)
            tot_xyz[i][j] = t_xyz1[i][j];
    }
    k = tn1;  /* total atom number in strand I */

    /* The order of strand II is now from 3'--->5', this need
       to be reversed according to information in <ts2> */
    for (i = nbp; i >= 1; i--) {
        for (j = ts2[i][1]; j <= ts2[i][1] + ts2[i][2] - 1; j++) {
            m = k + j - ts2[i][1] + 1;
            strcpy(tot_asym[m], t_asym2[j]);
            base_type[m] = t_btype2[j];
            strand[m] = 'B';
            base_num[m] = t_bnum2[j];
            for (i2 = 1; i2 <= 3; i2++)
                tot_xyz[m][i2] = t_xyz2[j][i2];
        }
        k += ts2[i][2];
    }

    /* Align the helix */
    align_helix(tot_xyz2, tot_xyz, tot_num, org_xyz, nbp);

    /* Write out the result */
    wrtpdb(tot_num, tot_asym, base_type, strand, base_num, tot_xyz2, filnam);

    /* free memory */
    free_cvector(dir_name, 1, BUF512);
    free_cvector(t_btype1, 1, MAX_ATOM);
    free_lvector(t_bnum1, 1, MAX_ATOM);
    free_cvector(t_btype2, 1, MAX_ATOM);
    free_lvector(t_bnum2, 1, MAX_ATOM);
    free_dvector(bp_pos, 1, 3);
    free_dvector(tmp_vec, 1, 3);
    free_dvector(hinge, 1, 3);
    free_cvector(base_type, 1, tot_num);
    free_cvector(strand, 1, tot_num);
    free_lvector(base_num, 1, tot_num);
    free_dmatrix(org_xyz, 1, nbp, 1, 3);
    free_cmatrix(t_asym1, 1, MAX_ATOM, 1, 4);
    free_dmatrix(t_xyz1, 1, MAX_ATOM, 1, 3);
    free_cmatrix(t_asym2, 1, MAX_ATOM, 1, 4);
    free_dmatrix(t_xyz2, 1, MAX_ATOM, 1, 3);
    free_lmatrix(ts2, 1, nbp, 1, 2);
    free_dmatrix(v_orien, 1, 3, 1, 3);
    free_dmatrix(v_orien_T, 1, 3, 1, 3);
    free_dmatrix(bp_orien, 1, 3, 1, 3);
    free_dmatrix(bp_orien_T, 1, 3, 1, 3);
    free_dmatrix(tot_xyz, 1, tot_num, 1, 3);
    free_dmatrix(tot_xyz2, 1, tot_num, 1, 3);
    free_cmatrix(tot_asym, 1, tot_num, 1, 4);
}
